//---------------------------------------------------------------------------
/*
 * Copyright (c) Allan Petersen, 2003.
 * This is a tutorial for twain scanning example
 *
 * (c) Copyright 2003, Allan Petersen
 * ALL RIGHTS RESERVED
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that
 * the name of Allan Petersen not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission.
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL ALLAN
 * PETERSEN BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT ALLAN PETERSEN HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Contact Allan Petersen at <support@allanpetersen.com> or visit
 * www.allanpetersen.com
 *
 *  My main mdi section of this program. (AP)
 *
 */
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "TwainCpp.h"
#include "Main.h"
#include "About.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainTwain::TMainTwain (HWND hWnd) : CTwain (hWnd)
{
}
//---------------------------------------------------------------------------
int TMainTwain::GetPaletteSize(TW_IMAGEINFO& bmInfo)
{
	switch(bmInfo.BitsPerPixel)
	{
	case 1:
			return 2;
	case 4:
			return 16;
	case 8:
			return 256;
	default:
			return 0;
	}
}
//---------------------------------------------------------------------------
//  This copies the scanned data to a bitmap.
//  and creates a new window
//---------------------------------------------------------------------------
void TMainTwain::CopyImage(HANDLE hBitmap,TW_IMAGEINFO& info)
{
    int    BitMapInfoSize,BitMapImageSize;
    void              *BitMapImage;
    BITMAPINFO        *BitMapInfo;
    TMDIChild         *Child;
    RECT               Rect;
    BITMAPINFO header;
    int        Bytes, ii, size;
    UCHAR       *lpVoid,
                *pBits;
    LPBITMAPINFOHEADER pHead;
    Graphics::TBitmap *pBitmap;
    Byte        *ptr1;
    Byte        *ptr2;

    Child = MainForm->CreateMDIChild("Image" + IntToStr(MainForm->MDIChildCount + 1));


	lpVoid = (UCHAR *)GlobalLock(hBitmap);
	pHead = (LPBITMAPINFOHEADER )lpVoid;


	Bytes = (pHead->biBitCount*pHead->biWidth) >>3;
	while(Bytes%4) Bytes++;

	pBits = lpVoid + sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD)*GetPaletteSize(info);


    // ---- Later set the appropiate pixel format based on
    // ---- the Bitmap info header, but, be aware of the
    // ---- data sizes in the copy routine.
    pBitmap = new Graphics::TBitmap;
    pBitmap->PixelFormat  = pf24bit;
    pBitmap->Width  = pHead->biWidth;
    pBitmap->Height = pHead->biHeight;


    // ---- running backwards to avoid mirroring the image
    /*
    for (ii=pHead->biHeight-1; ii>=0; ii--) {
        ptr1 = (Byte *) pBitmap->ScanLine[ii];
        memcpy (ptr1, pBits, Bytes);
        pBits += Bytes;
        }
        */

    // ---- Upgraded to handle gray scaled images
    // ---- Thanx to Don Nell.
    SetDIBits(pBitmap->Canvas->Handle,// handle of device context
              pBitmap->Handle,	      // handle of bitmap
              0,			          // starting scan line
              (UINT)pHead->biHeight,  // number of scan lines
              pBits,	              // array of bitmap bits
              (LPBITMAPINFO)lpVoid,	  // address of structure with bitmap data
              DIB_RGB_COLORS	 	  // type of color indices to use
              );
              
    Child->Image1->Picture->Bitmap->Assign (pBitmap);

    delete pBitmap;
	GlobalUnlock(lpVoid);

    Child->WHRatio = (float) pHead->biWidth/ (float) pHead->biHeight;
    Child->FormResize (NULL);
}
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent *Owner)
	: TForm(Owner)
{
    pCTwain = new TMainTwain(Handle);
}
//---------------------------------------------------------------------------
TMDIChild * __fastcall TMainForm::CreateMDIChild(String Name)
{
	TMDIChild *Child;

	//--- create a new MDI child window ----
	Child = new TMDIChild(Application);
	Child->Caption = Name;
    if (FileExists (Name)) {
		Child->Image1->Picture->Bitmap->LoadFromFile (Name);
        Child->WHRatio = (float)Child->Image1->Picture->Bitmap->Width / (float)Child->Image1->Picture->Bitmap->Height;
        }
    return (Child);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileNew1Execute(TObject *Sender)
{
	CreateMDIChild("Image" + IntToStr(MDIChildCount + 1));
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FileOpen1Execute(TObject *Sender)
{
	if (OpenDialog->Execute())
		CreateMDIChild(OpenDialog->FileName);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::HelpAbout1Execute(TObject *Sender)
{
	AboutBox->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FileExit1Execute(TObject *Sender)
{
	Close();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SelectSource1Click(TObject *Sender)
{
    pCTwain->SelectSource();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
    delete pCTwain;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Aquire1Click(TObject *Sender)
{
    // ---- TWCPP_ANYCOUNT fails on some systems.
    // pCTwain->Acquire(TWCPP_ANYCOUNT);
    pCTwain->Acquire(1);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ApplicationEvents1Message(tagMSG &Msg,
      bool &Handled)
{
    pCTwain->ProcessMessage(Msg);    
}
//---------------------------------------------------------------------------















